package com.castledragmire.GPSMapping;

//GUI
import android.os.Bundle;
import android.app.Activity;
import android.content.Intent;
import android.os.Handler;
import java.text.DateFormat;
import java.util.Date;
//Net
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.InputStreamReader;
import java.net.URL;
//GPS
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.location.LocationProvider;
//Settings
import java.security.MessageDigest;
import android.content.SharedPreferences;
import java.util.Locale;
import java.math.BigInteger;

public class GPSMappingService extends android.app.Service implements LocationListener
{
	//Interact with the GUI
	static public GPSMapping GPSMappingGUI=null;
	static public String LastUpdate="Could not start GPS Listening Service";
	static private GPSMappingService Me=null; //Remember service so GUI knows if it exists or needs to start it
	static private Intent MyService=null;
	//GPS
	private LocationManager LM;
	//Settings
	private String UserName="", SendDataURL="";
	private String Password="", MinTime="", MinDistance="";

	//Service functions
	@Override public void onCreate()
	{
		super.onCreate();
		Me=this;

		SetGUIText("Service Initialized");
		SenderThread=new Thread(RunSenderThread=new SenderThreadClass());
		SenderThread.start();
		LM=(LocationManager)getSystemService(LOCATION_SERVICE);
		UpdateVariablesLocal();
	}
	@Override public void onDestroy()
	{
		SetGUIText("Service disabled");
		synchronized(RunSenderThread)
		{
			RunSenderThread.CloseThread=true;
			RunSenderThread.notify();
		}
		LM.removeUpdates(this);
		Me=null;
		MyService=null;
		SenderThread=null;
		super.onDestroy();
	}
	@Override public android.os.IBinder onBind(android.content.Intent intent) { return null; }

	//Settings
	private void UpdateVariablesLocal() //Load the settings and update the location manager if needed
	{
		//Load normal string text settings
		SharedPreferences MyPrefs=getSharedPreferences("MyPrefs", 0);
		UserName=MyPrefs.getString("UserName", "");
		SendDataURL=MyPrefs.getString("SendDataURL", "");

		//Hash (SHA1) the password
		Password=UserName+"!"+MyPrefs.getString("Password", "");
		try { Password=new BigInteger(1, MessageDigest.getInstance("SHA1").digest(Password.getBytes())).toString(16).toUpperCase(Locale.US); } catch(Exception E) { Password=""; }

		//Refresh GPS location updates if min time or distance has changed
		String NewMinTime=MyPrefs.getString("MinTime", ""), NewMinDistance=MyPrefs.getString("MinDistance", "");
		if(!MinTime.equals(NewMinTime) || !MinDistance.equals(NewMinDistance))
			LM.requestLocationUpdates(LocationManager.GPS_PROVIDER, Integer.parseInt(MinTime=NewMinTime)*1000, Integer.parseInt(MinDistance=NewMinDistance), this); //Location updates depending on variables (every x second and minimum of y meter)
		onLocationChanged(LM.getLastKnownLocation(LocationManager.GPS_PROVIDER)); //Resend location
	}
	public static void UpdateVariables(boolean NeedsUpdate, Activity A) //Create the service or update its settings
	{
		//Just update the variables if the service already exists
		if(Me!=null)
		{
			if(NeedsUpdate)
				Me.UpdateVariablesLocal();
			return;
		}

		//Start the service or settings
		SharedPreferences MyPrefs=A.getSharedPreferences("MyPrefs", 0);
		if(MyPrefs.getBoolean("AllSettingsValid", false) && MyPrefs.getBoolean("IsEnabled", true)) //Start the service
			try {
				A.startService(MyService=new Intent(A, GPSMappingService.class)); }
			catch(Exception E)
			{
				SetGUIText("Cannot start GPS Mapping Service: "+E.getMessage());
				MyService=null;
			}
		else
		{
			SetGUIText("Service not started: "+(!MyPrefs.getBoolean("IsEnabled", true) ? "Not enabled" : "Invalid settings"));
			if(!NeedsUpdate && (MyPrefs.getString("UserName", "").equals("") || MyPrefs.getString("Password", "").equals(""))) //Open settings dialog if UserName or password is not set and we aren't coming from the settings dialog
				A.startActivity(new Intent(A, Settings.class));
		}
	}
	public static void StopService(Activity A)
	{
		if(Me==null)
			return;
		A.stopService(MyService);
	}

	//Interact with GUI
	private static void SetGUIText(String TheText)
	{
		LastUpdate="Time: "+DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM).format(new Date())+'\n'+TheText;
		if(GPSMappingGUI!=null)
			GPSMappingGUI.CurrentPosition.setText(LastUpdate);
	}

	//GPS Functions
	private int NumberOfUpdates=0;
	@Override public void onLocationChanged(Location location) //Handle a location change
	{
		if(location==null) //Check for unknown location
			return;

		//Set the GUI Text
		String Output=
			"Update Number: "+(++NumberOfUpdates)+'\n'+
			"Latitude: "+location.getLatitude()+'\n'+
			"Longitude: "+location.getLongitude()+'\n'+
			"Altitude: "+location.getAltitude()+'\n'+
			"Accuracy: "+location.getAccuracy()+'\n';
		SetGUIText(Output);

		//Compile the message to send
		String CompiledSendDataURL=SendDataURL+"?"+
			"Latitude="+location.getLatitude()+'&'+
			"Longitude="+location.getLongitude()+'&'+
			"Altitude="+location.getAltitude()+'&'+
			"Accuracy="+location.getAccuracy()+'&'+
			"Time="+location.getTime()+'&'+
			"UserName="+UserName+'&'+
			"Password="+Password;

		//Send it to the thread
		synchronized(RunSenderThread)
		{
			RunSenderThread.DataToSend=CompiledSendDataURL;
			RunSenderThread.CurrentOutput=Output;
			RunSenderThread.notify();
		}
	}

	//When the service is running, this thread is open to communicate with the server (Since starting in gingerbread the main thread cannot do network requests)
	private Thread SenderThread;
	private SenderThreadClass RunSenderThread;
	private class SenderThreadClass implements Runnable
	{
		//These variables should only be changed when locked (synchronized) with this object 
		public Boolean CloseThread=false; //This is set to true when the service is ending to stop this thread
		public String DataToSend; //When there is data ready to be sent, this is filled. Otherwise it is null. If a piece of data hasn't been sent when the next is ready, it is overwritten
		public String CurrentOutput; //The current output on the GUI for the DataToSend
		public Handler SetTextHandler=new Handler();
		
		@Override public void run()
		{
			android.os.Looper.prepare();
			while(true)
			{
				//Check the synchronized variables
				String Data, Output;
				synchronized(this)
				{
					if(CloseThread) //If service is stopping, kill the thread
						return;
					if(DataToSend==null) //If there is no data to send, wait for a signal that there is
					{
						try { this.wait(); } catch (InterruptedException e) {}
						continue;
					}

					//If there is data to send, store it locally and wipe it from the synchronized objects 
					Data=DataToSend;
					Output=CurrentOutput;
					DataToSend=CurrentOutput=null;
				}

				//Communicate the data to the server
				try
				{
					//Receive the response
					BufferedReader SendData=new BufferedReader(new InputStreamReader((new URL(Data)).openStream()));
					String Line="";
					if((Line=SendData.readLine())==null || !Line.equals("MAPPING")) //Make sure the response is from the mapping API
						Output+="\nThe URL you specified in the settings is not pointing to a valid GPS Mapping API";
					else //Store the response and add it to the output on the GUI
					{
						String ServerOutput="";
						while((Line=SendData.readLine())!=null)
							ServerOutput+='\n'+Line;
						Output+="\nResponse from server: "+ServerOutput;
					}
				} catch(FileNotFoundException E) {
					Output+="\nProblem communicating with server: Invalid URL";
				} catch(Exception E) {
					Output+="\nProblem communicating with server: "+E.toString();
				}

				//Output the additional server response
				final String SendOutput=Output;
				SetTextHandler.post(new Runnable() { @Override public void run() {
					SetGUIText(SendOutput);					
				}});
			}
		}
	}

	@Override public void onProviderDisabled(String provider) //Called when GPS is disabled
	{
		SetGUIText("GPS Disabled");
		startActivity(new Intent(android.provider.Settings.ACTION_LOCATION_SOURCE_SETTINGS).setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)); //Bring up the GPS settings
	}

	@Override public void onProviderEnabled(String provider) //Called when GPS is enabled
	{
		SetGUIText("GPS Enabled");
	}

	@Override public void onStatusChanged(String provider, int status, Bundle extras) //Called when the GPS status alters
	{
		switch (status)
		{
			case LocationProvider.OUT_OF_SERVICE:			SetGUIText("Status Changed: Out of Service");			break;
			case LocationProvider.TEMPORARILY_UNAVAILABLE:	SetGUIText("Status Changed: Temporarily Unavailable");	break;
			case LocationProvider.AVAILABLE:																		break; //SetGUIText("Status Changed: Available"); break;
		}
	}
}