/***Copyright and coded by Dakusan - See http://www.castledragmire.com/Copyright for more information. ***/
/***Dakusan's AJAX Library - v1.0.1 http://www.castledragmire.com/Projects/Other_Web_Scripts#AJAX ***/

function GetURL(	//This function calls a web address through AJAX and sends it data. If the call is made synchronously (see CallBack option), the URL's output is returned, otherwise, the AJAX object is returned.
	URL,		//The URL of the address to request. If the document is not found, an error is thrown.
	Data,		/*The data to send in the query section of the URL.
				Value types:
					String: Uses the string as the URL's query if using URL Encoding (“application/x-www-form-urlencoded”). Otherwise ("multipart/form-data" encoding), this is set to the parameter "NoName".
					Object: Uses the object's members as parameters. The members' values must be scalars.
				Other Information: This parameter is not optional. Set to a blank string to send nothing.*/
	ExtraOptions)	//An optional object specifying additional options. See first section below for possible members
{
	//Set additional options
	var Options=
	{
		CallBack:undefined,		//Function to call when the document has completed. It receives 3 parameters: The responseText, the ExtraData additional option, and the AJAX object used to make the call. If this is not set, the AJAX call is made synchronously. Pass false to call asynchronously without a callback function.
		ExtraData:undefined,		//User defined data passed directly to the CallBack function
		UseCache:true,			//If this is true, the request is made via GET with URL encoding ("application/x-www-form-urlencoded"), so timestamps are sent meaning the browser cache can be used. Otherwise, POST is used with "multipart/form-data" encoding (unless it takes less bytes to send as "application/x-www-form-urlencoded" in the POST).
		ForceUrlEncode:undefined,	//If set and POSTing, "application/x-www-form-urlencoded" encoding is used when true, and "multipart/form-data" when false, otherwise the type is used that yields the smallest size. (It can be expensive to determine "multipart/form-data" encoding if it is not needed).
		AdditionalHeaders:undefined	//A list of additional HTTP headers to pass to the request in an object (members as variable names with scalars values only)
	};
	if(ExtraOptions instanceof Object) //Copy over passed additional options to the options object
		for(var i in ExtraOptions)
			if(ExtraOptions.hasOwnProperty(i))
				Options[i]=ExtraOptions[i];

	//Create the AJAX object
	var Async=(typeof(Options.CallBack)=="function" || Options.CallBack===false); //Determine if this is to be done asynchronously
	var PostObj = window.XMLHttpRequest ? new XMLHttpRequest() : new ActiveXObject("Microsoft.XMLHTTP"); //Create the AJAX object
	if(Async) //If running asynchronously, prepare the AJAX callback when the request has completed
		PostObj.onreadystatechange=function()
		{
			if(PostObj.readyState==4) //If the call has finished...
				if(Options.CallBack===false) //If no callback...
					{}
				else if(PostObj.status<400) //If there is a callback and the request completed sucessfully
					Options.CallBack(PostObj.responseText, Options.ExtraData, PostObj);
				else //If the request was completed unsucessfully
					GetURL_Error(PostObj, ExtraData); //Throw an error
		};

	//Prepare the data to send - the data is initially set to assume that UseCache is true
	var NewData=GetURL_Encoding_URLEncoded(Data); //The data that we will be sending
	var EncodingType='application/x-www-form-urlencoded; charset=utf-8;'; //The data encoding type
	if(!Options.UseCache && Options.ForceUrlEncode!==true) //If we are not using the cache, see if the data can be encoded better through multipart/form-data
	{
		var FormDataEncoding=GetURL_Encoding_FormDataEncoded(Data);
		if(Options.ForceUrlEncode===false || FormDataEncoding.Data.length<NewData.length) //If it takes less bytes to send as form-data instead of urlencoded, use form-data
		{
			NewData=FormDataEncoding.Data;
			EncodingType='multipart/form-data; boundary='+FormDataEncoding.Boundary;
		}
	}

	//Send the data
	PostObj.open(Options.UseCache ? 'GET' : 'POST', URL+(Options.UseCache && NewData!='' ? '?'+NewData : ''), Async); //Prepare the connection to the URL
	if(!Options.UseCache)
		PostObj.setRequestHeader('Content-Type', EncodingType); //Set the encoding type
	if(Options.AdditionalHeaders instanceof Object) //Send additional headers
		for(var i in Options.AdditionalHeaders)
			if(Options.AdditionalHeaders.hasOwnProperty(i))
				PostObj.setRequestHeader(i, Options.AdditionalHeaders[i]);
	PostObj.send(Options.UseCache ? '' : NewData); //Initialize AJAX call

	//Complete the function
	if(!Async && PostObj.status>=400) //Check for errors
		GetURL_Error(PostObj, ExtraData); //If the URL is not found, throw an error
	return Async ? PostObj : PostObj.responseText; //Return either the response text or the AJAX object depending on if the call is asynchronous or not
}

function GetURL_Encoding_URLEncoded(Data) //Compile the data as application/x-www-form-urlencoded
{
	if(!(Data instanceof Object)) //If not an object, just send the scalar
		return (Data===undefined ? 'undefined' : Data===null ? 'null' : Data.toString());

	//Send each member as a variable
	var NewData=[];
	for(var i in Data)
		NewData.push(escape(i)+"="+encodeURIComponent(Data[i].toString()));
	return NewData.join("&");
}

function GetURL_Encoding_FormDataEncoded(Data) //Compile the data as multipart/form-data
{
	if(!(Data instanceof Object)) //If an object is not passed, turn the data into an object with it set to the member name "NoName"
		Data={NoName:(Data===undefined ? 'undefined' : Data===null ? 'null' : Data.toString())};

	//Encode each section
	var NewData=[];
	for(var i in Data)
		NewData.push('Content-Disposition: form-data; name="'+i+"\"\r\n\r\n"+Data[i].toString());

	//Determine a random boundary
	var Boundary;
	while(true)
	{
		//Create a random 6 character alphanumeric boundary
		Boundary='--';
		for(var i=0;i<6;i++)
		{
			var RandomChar;
			while((RandomChar=String.fromCharCode(parseInt(Math.random()*74+48))).match(/[A-Za-z0-9]/)===null); //Get a random alpha numeric character
			Boundary+=RandomChar; //Append the character to the boundary
		}

		//Make sure the boundary doesn't interfere with any of the items
		for(var i=0;i<NewData.length;i++)
			if(NewData[i].indexOf(Boundary)!=-1)
				break;
		if(i==NewData.length) //If the boundary wasn't used in any of the strings, it is valid, so exit out of the loop to find a boundary
			break;
	}

	return {Data:Boundary+"\r\n"+NewData.join("\r\n"+Boundary+"\r\n")+"\r\n"+Boundary+'--'+"\r\n", Boundary:Boundary.substr(2)}; //Return the compiled message and the boundary
}

function GetURL_Error(PostObj, ExtraData)
{
	throw('Error in requesting File through AJAX. Status code returned: '+PostObj.status, PostObj.responseText, PostObj);
}